"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResolveArg = void 0;
const ContainerArg_1 = require("./ContainerArg");
const is = require("predicates");
const isOmitableType = is.any(is.primitive, is.func);
/**
 * Resolves all ContainerArg in provided value
 */
class ResolveArg extends ContainerArg_1.ContainerArg {
    constructor(value) {
        super();
        this.value = value;
        this.reducer = (acc, current) => {
            const args = this.collectContainerArgsFromResult(current);
            if (args) {
                return acc.concat(args);
            }
            return acc;
        };
        Object.freeze(this);
    }
    getArgument(container) {
        return this.transformObject(this.getValue(), container);
    }
    getValue() {
        return is.func(this.value) ? this.value() : this.value;
    }
    async transformObject(result, container) {
        if (isOmitableType(result)) {
            return result;
        }
        if (ContainerArg_1.ContainerArg.is(result)) {
            return result.getArgument(container);
        }
        if (is.arr(result)) {
            return Promise.all(result.map(x => isOmitableType(x) ? x : this.transformObject(x, container)));
        }
        if (is.plainObject(result)) {
            const newObject = {};
            for (const [key, value] of Object.entries(result)) {
                newObject[key] = isOmitableType(value) ? value : await this.transformObject(value, container);
            }
            return newObject;
        }
        return result;
    }
    collectContainerArgsFromResult(result) {
        if (ContainerArg_1.ContainerArg.is(result)) {
            return [result];
        }
        if (is.primitive(result)) {
            return;
        }
        if (is.array(result)) {
            return result.reduce(this.reducer, []);
        }
        if (is.plainObject(result)) {
            return Object.values(result)
                .reduce(this.reducer, []);
        }
    }
    getDependentServices(container) {
        const value = this.getValue();
        const args = this.collectContainerArgsFromResult(value);
        if (args) {
            return args.reduce((acc, current) => {
                const deps = current.getDependentServices(container);
                return acc.concat(Array.isArray(deps) ? deps : [deps]);
            }, []);
        }
        return [];
    }
}
exports.ResolveArg = ResolveArg;
